package com.jz.common.utils;


import org.apache.commons.io.IOUtils;
import org.apache.commons.io.output.FileWriterWithEncoding;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import java.io.*;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 删除文件和目录
 *
 */
public class FileUtils {
	
	private static final Logger logger = LoggerFactory.getLogger(FileUtils.class);
	
	/**
	 * 写文件
	 * @param fileAbsolutePath 文件绝对路径
	 * @param fileContent 要写入的文件内容
	 */
	
	@SuppressWarnings("unchecked")
	public static void write(String fileAbsolutePath, Object fileContentObject) {
		FileWriterWithEncoding fileWriter = null;
		BufferedWriter bufferedWriter = null;
		try {
			fileWriter = new FileWriterWithEncoding(fileAbsolutePath, "UTF-8") ;
			bufferedWriter = new BufferedWriter(fileWriter);
			if (fileContentObject instanceof String) {
				String fileContent = (String) fileContentObject;
				bufferedWriter.write(fileContent);
			}else if ( fileContentObject instanceof List) {
				List<String> fileContentList = (List<String>) fileContentObject;
				for (String fileContent : fileContentList) {
					bufferedWriter.write(fileContent+"\n");
					bufferedWriter.flush();
				}
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			throw new RuntimeException(fileAbsolutePath+" 写文件异常");
		}catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException(fileAbsolutePath+" 写文件异常");
		}finally {
			if (bufferedWriter != null) {
				try {
					bufferedWriter.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if (fileWriter != null) {
				try {
					fileWriter.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			
			
		}
		
	}

    /**
     * 删除文件，可以是文件或文件夹
     *
     * @param fileName
     *            要删除的文件名
     * @return 删除成功返回true，否则返回false
     */
    public static boolean delete(String fileName) {
        File file = new File(fileName);
        if (!file.exists()) {
            logger.info("删除文件失败:" + fileName + "不存在！");
            return false;
        } else {
            if (file.isFile())
                return deleteFile(fileName);
            else
                return deleteDirectory(fileName);
        }
    }

    /**
     * 删除单个文件
     *
     * @param fileName
     *            要删除的文件的文件名
     * @return 单个文件删除成功返回true，否则返回false
     */
    public static boolean deleteFile(String fileName) {
        File file = new File(fileName);
        // 如果文件路径所对应的文件存在，并且是一个文件，则直接删除
        if (file.exists() && file.isFile()) {
            if (file.delete()) {
                logger.info("删除单个文件" + fileName + "成功！");
                return true;
            } else {
                logger.info("删除单个文件" + fileName + "失败！");
                return false;
            }
        } else {
            logger.info("删除单个文件失败：" + fileName + "不存在！");
            return false;
        }
    }

    /**
     * 删除目录及目录下的文件
     *
     * @param dir
     *            要删除的目录的文件路径
     * @return 目录删除成功返回true，否则返回false
     */
    public static boolean deleteDirectory(String dir) {
        // 如果dir不以文件分隔符结尾，自动添加文件分隔符
        if (!dir.endsWith(File.separator))
            dir = dir + File.separator;
        File dirFile = new File(dir);
        // 如果dir对应的文件不存在，或者不是一个目录，则退出
        if ((!dirFile.exists()) || (!dirFile.isDirectory())) {
            logger.info("删除目录失败：" + dir + "不存在！");
            return false;
        }
        boolean flag = true;
        // 删除文件夹中的所有文件包括子目录
        File[] files = dirFile.listFiles();
        for (int i = 0; i < files.length; i++) {
            // 删除子文件
            if (files[i].isFile()) {
                flag = FileUtils.deleteFile(files[i].getAbsolutePath());
                if (!flag)
                    break;
            }
            // 删除子目录
            else if (files[i].isDirectory()) {
                flag = FileUtils.deleteDirectory(files[i]
                        .getAbsolutePath());
                if (!flag)
                    break;
            }
        }
        if (!flag) {
            logger.info("删除目录失败！");
            return false;
        }
      return true;
    }
    
    /**
     * 拷贝目录
     * @param sourceDir 源目录
     * @param destinationDir 目标目录
     * @throws IOException
     */
    public static void copyDir(String sourceDir, String destinationDir){
        File file = new File(sourceDir);
        String[] filePath = file.list();
        
        if (!(new File(destinationDir)).exists()) {
            (new File(destinationDir)).mkdir();
        }
        
        for (int i = 0; i < filePath.length; i++) {
            if ((new File(sourceDir + "/" + filePath[i])).isDirectory()) {
                copyDir(sourceDir  + "/"  + filePath[i], destinationDir  + "/" + filePath[i]);
            }
            if (new File(sourceDir  + "/" + filePath[i]).isFile()) {
                copyFile(sourceDir + "/" + filePath[i], destinationDir + "/" + filePath[i]);
            }
            
        }
        
    }
    
    /**
     * 拷贝文件
     * @param sourceFilePath 源文件路劲
     * @param destinationFilePath 目标文件路径
     * @throws IOException
     */
    public static void copyFile(String sourceFilePath, String destinationFilePath) {
        File sourceFile = new File(sourceFilePath);
        File destinationFile = new File(destinationFilePath);
        
        FileInputStream in = null ;
        
        FileOutputStream out = null ;
		try {
			in = new FileInputStream(sourceFile);
			
			out = new FileOutputStream(destinationFile);
			
			try {
				IOUtils.copy(in, out);
			} catch (IOException e) {
				e.printStackTrace();
			}
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		finally {
			if (in != null) {
				try {
					in.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			
			if (out != null ) {
				try {
					out.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
        
        
    }
    
    /**
     * 列出文件名
     * @param dir 要列出的目录
     * @param suffix 要过滤的文件名后缀
     * @return
     */
    public static Map<String, Integer> listDirFiles (String dir,String suffix){
    	
    	if (StringUtils.isEmpty(dir)) {
			return null;
		}
    	File  file = new File(dir);
    	if (file.exists()) {
    		
    		Map<String, Integer> map = new HashMap<>();
    		
			File[] listFiles = file.listFiles();
			for (File file2 : listFiles) {
				 String fileName = file2.getName();
				 if (fileName.endsWith(suffix)) {
					fileName = fileName.substring(0,fileName.lastIndexOf(suffix));
					map.put(fileName, 1);
				}
			}
			return map;
		}
    	return null;
    }
    
    
    

    public static void main(String[] args) throws NoSuchFieldException, SecurityException {
		//  // 删除单个文件
		//  String file = "c:/test/test.txt";
		//  DeleteFileUtil.deleteFile(file);
		//  logger.info();
        // 删除一个目录
        //String dir = "D:/1/aa.sh";
        //FileUtils.deleteDirectory(dir);
		//  logger.info();
		//  // 删除文件
		//  dir = "c:/test/test0";
		//  DeleteFileUtil.delete(dir);
        /*List<String> list = new ArrayList<>();
        list.add("#!/bin/sh");
		list.add("echo 123");
		list.add("sleep 5000");
        write(dir, list);*/
    	
    	/*String sourceDir = "d:/source/zhangsad/182";
    	
    	String destinationDir = "d:/target";
    	
    	
    	copyDir(destinationDir,sourceDir);*/
    	
    	Map<String, Integer> listDirFiles = listDirFiles("D:\\10\\temp\\111\\55", ".job");
    	System.err.println(listDirFiles);
    }
    
    

}